# - Config file for the Palisade package
# It defines the following variables
#  PALISADE_INCLUDE_DIRS - include directories for Palisade
#  PALISADE_LIBRARIES    - libraries to link against

get_filename_component(PALISADE_CMAKE_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

# Our library dependencies (contains definitions for IMPORTED targets)
if(NOT Palisade_BINARY_DIR)
  include("${PALISADE_CMAKE_DIR}/PalisadeTargets.cmake")
endif()

# These are IMPORTED targets created by PalisadeTargets.cmake
set(PALISADE_INCLUDE "/usr/local/include/palisade")
set(PALISADE_LIBDIR "/usr/local/lib")
set(PALISADE_LIBRARIES PALISADEcore;PALISADEpke;PALISADEbinfhe  -fopenmp=libomp)
set(PALISADE_STATIC_LIBRARIES   -fopenmp=libomp)
set(PALISADE_SHARED_LIBRARIES PALISADEcore;PALISADEpke;PALISADEbinfhe  -fopenmp=libomp)
set(BASE_PALISADE_VERSION 1.11.9)


set(OPENMP_INCLUDES "" )
set(OPENMP_LIBRARIES "" )

set(PALISADE_CXX_FLAGS "-O2 -pipe -fstack-protector-strong -isystem /usr/local/include -fno-strict-aliasing  -I/wrkdirs/usr/ports/security/palisade/work/palisade-release-3d1f9a3f9fd389df1f8f27f286511f4f5871258b/third-party/google-test/googletest -Wno-error=unused-but-set-variable -isystem /usr/local/include -Wall  -DPALISADE_VERSION=1.11.9  -Wno-unused-private-field -Wno-shift-op-parentheses -DMATHBACKEND=2 -fopenmp=libomp -fopenmp=libomp")
set(PALISADE_C_FLAGS "-O2 -pipe  -fstack-protector-strong -isystem /usr/local/include -fno-strict-aliasing -Wall  -DPALISADE_VERSION=1.11.9 -DMATHBACKEND=2 -fopenmp=libomp -fopenmp=libomp")

if( "OFF" STREQUAL "Y" )
	set(PALISADE_CXX_FLAGS "${PALISADE_CXX_FLAGS} -DWITH_NTL" )
	set(PALISADE_C_FLAGS "${PALISADE_C_FLAGS} -DWITH_NTL")
endif()

set (PALISADE_EXE_LINKER_FLAGS " -pthread -fstack-protector-strong  ")

# CXX info
set(PALISADE_CXX_STANDARD "11")
set(PALISADE_CXX_COMPILER_ID "Clang")
set(PALISADE_CXX_COMPILER_VERSION "18.1.5")

# Build Options
set(PALISADE_STATIC "OFF")
set(PALISADE_SHARED "ON")
set(PALISADE_TCM "OFF")
set(PALISADE_WITH_INTEL_HEXL "OFF")
set(PALISADE_OPENMP "true")
set(PALISADE_NATIVE_SIZE "64")
set(PALISADE_CKKS_M_FACTOR "1")
set(PALISADE_NATIVEOPT "OFF")

# Math Backend
if("ON")
	set(PALISADE_BACKEND "BE2")
elseif("ON")
	set(PALISADE_BACKEND "BE4")
elseif("OFF")
	set(PALISADE_BACKEND "NTL")
endif()

# Build Details
set(PALISADE_EMSCRIPTEN "")
set(PALISADE_ARCHITECTURE "amd64")
set(PALISADE_BACKEND_FLAGS_BASE "-DMATHBACKEND=2")

# Compile Definitions

if( "ON" )
	set(PALISADE_BINFHE_COMPILE_DEFINITIONS "_compile_defs-NOTFOUND")
	set(PALISADE_CORE_COMPILE_DEFINITIONS "_compile_defs-NOTFOUND")
	set(PALISADE_PKE_COMPILE_DEFINITIONS "_compile_defs-NOTFOUND")
	set(PALISADE_COMPILE_DEFINITIONS
			${PALISADE_BINFHE_COMPILE_DEFINITIONS}
			${PALISADE_CORE_COMPILE_DEFINITIONS}
			${PALISADE_PKE_COMPILE_DEFINITIONS})
endif()

if( "OFF" )
	set(PALISADE_BINFHE_COMPILE_DEFINITIONS_STATIC "")
	set(PALISADE_CORE_COMPILE_DEFINITIONS_STATIC "")
	set(PALISADE_PKE_COMPILE_DEFINITIONS_STATIC "")
	set(PALISADE_COMPILE_DEFINITIONS_STATIC
			${PALISADE_BINFHE_COMPILE_DEFINITIONS_STATIC}
			${PALISADE_CORE_COMPILE_DEFINITIONS_STATIC}
			${PALISADE_PKE_COMPILE_DEFINITIONS_STATIC})
endif()
